@@ -666,43 +666,6 @@ selection_changed (GtkTreeSelection *tree_selection,
         g_free (attr_name);
 }
 
-static gchar *
-get_grey_color (GtkTreeView *tree_view,
-                GtkTreeIter *iter)
-{
-        GtkTreeSelection *tree_selection;
-        GtkStyle *style;
-        GdkColor desc_gdk_color = {0};
-        GtkStateType state;
-        gchar *desc_color;
-
-        /* This color business shouldn't be this hard... */
-        tree_selection = gtk_tree_view_get_selection (tree_view);
-        style = gtk_widget_get_style (GTK_WIDGET (tree_view));
-        if (gtk_tree_selection_iter_is_selected (tree_selection, iter)) {
-                if (GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (tree_view)))
-                        state = GTK_STATE_SELECTED;
-                else
-                        state = GTK_STATE_ACTIVE;
-        } else {
-                state = GTK_STATE_NORMAL;
-        }
-#define BLEND_FACTOR 0.7
-        desc_gdk_color.red   = style->text[state].red   * BLEND_FACTOR +
-                               style->base[state].red   * (1.0 - BLEND_FACTOR);
-        desc_gdk_color.green = style->text[state].green * BLEND_FACTOR +
-                               style->base[state].green * (1.0 - BLEND_FACTOR);
-        desc_gdk_color.blue  = style->text[state].blue  * BLEND_FACTOR +
-                               style->base[state].blue  * (1.0 - BLEND_FACTOR);
-#undef BLEND_FACTOR
-        desc_color = g_strdup_printf ("#%02x%02x%02x",
-                                      (desc_gdk_color.red >> 8),
-                                      (desc_gdk_color.green >> 8),
-                                      (desc_gdk_color.blue >> 8));
-
-        return desc_color;
-}
-
 static void
 format_markup_name (GtkCellLayout   *cell_layout,
                     GtkCellRenderer *renderer,
@@ -716,7 +679,6 @@ format_markup_name (GtkCellLayout   *cell_layout,
         gchar *name;
         gchar *desc;
         gchar *markup;
-        gchar *desc_color;
 
         gtk_tree_model_get (tree_model,
                             iter,
@@ -738,17 +700,31 @@ format_markup_name (GtkCellLayout   *cell_layout,
                 desc = g_strdup_printf (_("No description for attribute %d"), id);
         }
 
-        desc_color = get_grey_color (GTK_TREE_VIEW (dialog->priv->tree_view), iter);
         if (a->warn) {
                 markup = g_strdup_printf ("<b><span fgcolor='red'>%s</span></b>\n"
                                           "<span fgcolor='darkred'><small>%s</small></span>",
                                           name,
                                           desc);
         } else {
+                gchar color[16];
+                GtkTreeSelection *tree_selection;
+                GtkStateType state;
+
+                tree_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->priv->tree_view));
+                if (gtk_tree_selection_iter_is_selected (tree_selection, iter)) {
+                        if (gtk_widget_has_focus (GTK_WIDGET (dialog->priv->tree_view)))
+                                state = GTK_STATE_SELECTED;
+                        else
+                                state = GTK_STATE_ACTIVE;
+                } else {
+                        state = GTK_STATE_NORMAL;
+                }
+                gdu_util_get_mix_color (GTK_WIDGET (dialog->priv->tree_view), state, color, sizeof (color));
+
                 markup = g_strdup_printf ("<b>%s</b>\n"
                                           "<span fgcolor=\"%s\"><small>%s</small></span>",
                                           name,
-                                          desc_color,
+                                          color,
                                           desc);
         }
 
@@ -760,7 +736,6 @@ format_markup_name (GtkCellLayout   *cell_layout,
         g_free (name);
         g_free (desc);
         g_free (markup);
-        g_free (desc_color);
 }
 
 static void
